﻿using Microsoft.Extensions.Logging;
using PlumbingRepairContracts.BusinessLogicsContracts;
using PlumbingRepairContracts.SearchModels;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace PlumbingRepairView
{
    public partial class FormSellWork : Form
    {
        private readonly ILogger _logger;
        private readonly IWorkLogic _logicWork;
        private readonly IStoreLogic _logicStore;
        public FormSellWork(ILogger<FormSellWork> logger, IWorkLogic logicPackage, IStoreLogic logicStore)
        {
            InitializeComponent();
            _logger = logger;
            _logicWork = logicPackage;
            _logicStore = logicStore;
            LoadData();
        }

        private void FormSellWork_Load(object sender, EventArgs e)
        {
            LoadData();
        }

        private void LoadData()
        {
            _logger.LogInformation("Loading works for sale.");

            try
            {
                var list = _logicWork.ReadList(null);
                if (list != null)
                {
                    WorkСomboBox.DisplayMember = "WorkName";
                    WorkСomboBox.ValueMember = "Id";
                    WorkСomboBox.DataSource = list;
                    WorkСomboBox.SelectedItem = null;
                }
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "List loading error.");
                MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void SaveButton_Click(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(QuantityTextBox.Text))
            {
                MessageBox.Show("Укажите количество", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            if (WorkСomboBox.SelectedValue == null)
            {
                MessageBox.Show("Выберите изделие", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            _logger.LogInformation("Work sale.");

            try
            {
                var operationResult = _logicStore.SellWork(_logicWork.ReadElement(new WorkSearchModel()
                {
                    Id = Convert.ToInt32(WorkСomboBox.SelectedValue)
                })!, Convert.ToInt32(QuantityTextBox.Text));

                if (!operationResult)
                {
                    throw new Exception("Ошибка при продаже изделия. Дополнительная информация в логах.");
                }

                MessageBox.Show("Сохранение прошло успешно", "Сообщение", MessageBoxButtons.OK, MessageBoxIcon.Information);
                DialogResult = DialogResult.OK;

                Close();
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Work sale error.");
                MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void ButtonCancel_Click(object sender, EventArgs e)
        {
            DialogResult = DialogResult.Cancel;
            Close();
        }
    }
}
